#include <stdlib.h>
#include "modlist.h"

static intptr_t
rand_int(int isnegative) {
  intptr_t i = rand();
  if(isnegative) {
    if(i > 0) return -i;
    else return i;
  }
  else {
    if(i < 0) return -i;
    else return i;
  }
}

static intptr_t
rand_positive_int() {
  return rand_int(0);
}

static modref_t*
generate(int n) {
  return modlist_random(n, rand_positive_int);
}

static void
print_int(intptr_t x, FILE* f) {
  fprintf(f, "%d", (int)x);
}

static void
print(modref_t* modref, FILE* f) {
  modlist_print(modref, f, print_int);
}

static int equals(modref_t* m1, modref_t* m2) {
  return modlist_equals(m1, m2);
}

typedef struct iter_s {
  modref_t* modref;
} iter_t;

static void*
iter_new(modref_t* m) {
  iter_t* iter = malloc(sizeof(iter_t));
  iter->modref = m;
  return iter;
}

static void
iter_next(void* iter) {
  iter_t*      i = iter;
  modref_t*    m = i->modref;
  cons_cell_t* c = modref_deref(m);
  i->modref = c ? c->tl : NULL;
}

static int
iter_isdone(void* iter) {
  iter_t* i = iter;
  return i->modref == NULL;
}

static void
change_forw(void* iter) {
  iter_t*    i = iter;
  modref_t*  m = i->modref;
  modlist_insert(m, (void*) rand_int(1));
}

static void
change_back(void* iter) {
  iter_t*    i = iter;
  modref_t*  m = i->modref;
  modlist_remove(m);
}

test_input_hooks_t test_input_filter_sparse = {
  generate,
  print,
  iter_new,
  iter_next,
  iter_isdone,
  change_forw,
  change_back
};

test_output_hooks_t test_output_filter_sparse = {
  equals,
  print
};
